#!/bin/bash
#Description: 框架安装插件时调用
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-11-04 created

current_dir=$(readlink -f "$(dirname "$0")")
source "${current_dir}/utils.sh"
source "${current_dir}/doctor_cli.sh"
INSTALL_PATH="$1" # /opt/DevKit
PLUGIN_FLAG="$2" # web
PLUGIN="sys_diagnosis"


function init_variable() {
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    os_type=$(grep -w OS_TYPE "${CONFIG_FILE}" | awk -F= '{if($2 !="") print $2}')
    INSTALL_LOG_PATH="${INSTALL_PATH}/logs/sys_diagnosis_install.log"
    INSTALL_PLUGIN_PATH="${INSTALL_PATH}/devkitplugins/"
    PYTHON_PATH="${INSTALL_PATH}/tools/python_venv/bin/python3"
}

function create_check_pd_result() {
    # 创建 check_pd_result 文件 用于采集
    local check_pd_result_path="${INSTALL_PATH}/tools/rpc_cluster/check_pd_result"
    if [[ ! -f "${check_pd_result_path}" ]]; then
        touch "${check_pd_result_path}"
        chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${check_pd_result_path}"
        chmod 600 "${check_pd_result_path}"
    fi
}

function change_file_path_permission() {
    local diagnosis_path="${INSTALL_PLUGIN_PATH}/sys_diagnosis/"
    if [[ -d "${diagnosis_path}" ]]; then
        # 去除符合条件的文件的other权限
        find "${diagnosis_path}" -user "${DEVKIT_USER}" -type f -perm /007 -exec chmod o= {} \;
        find "${diagnosis_path}" -type d -exec chmod 700 {} \;
        chmod 755 "${diagnosis_path}"
        chown root:root "${diagnosis_path}"
    fi
    find "${diagnosis_path}/deploy" -type f -name "*\.sh" -exec chown root:root {} \; >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "*\.sh" -exec chmod 500 {} \; >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -regex ".*\.py\|.*\.pyc\|.*\.css\|.*\.js\|.*\.html\|.*\.png\|.*\.svg\|.*\.gif\|.*\.PNG\|.*\.ico\|.*\.md" -exec chmod 400 {} \; >> /dev/null 2>&1
    # 所有so权限更改为550
    find "${diagnosis_path}" -type f -name "*\.so" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "*\.so\.*" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1

    find "${diagnosis_path}" -type f -name "*\.ini" -print0 | xargs -0 chmod 600 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "*\.json" -print0 | xargs -0 chmod 400 >> /dev/null 2>&1
    find "${diagnosis_path}" -type f -name "Open_Source_Software_Notice\.txt" -print0 | xargs -0 chmod 400 >> /dev/null 2>&1
}

function load_data() {
    su - devkit -s /bin/bash -c "${PYTHON_PATH} ${INSTALL_PLUGIN_PATH}/manage.pyc loaddata \
      ${INSTALL_PLUGIN_PATH}/sys_diagnosis/system_config.json \
      ${INSTALL_PLUGIN_PATH}/sys_diagnosis/storage.json >> /dev/null"
    rm -f "${INSTALL_PLUGIN_PATH}/sys_diagnosis/system_config.json" "${INSTALL_PLUGIN_PATH}/sys_diagnosis/storage.json"
    echo_and_log "Load database success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
}

function create_export_import_dir() {
    local export_dir="${INSTALL_PATH}/tools/rpc_cluster/server/exports/"
    local import_dir="${INSTALL_PATH}/tools/rpc_cluster/server/imports/"
    local django_dir="${INSTALL_PATH}/tools/rpc_cluster/server/django/"
    if [[ ! -d "${export_dir}" ]]; then
         mkdir "${export_dir}"
         chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${export_dir}"
         chmod 700 "${export_dir}"
    fi
    if [[ ! -d "${import_dir}" ]]; then
         mkdir "${import_dir}"
         chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${import_dir}"
         chmod 700 "${import_dir}"
    fi
    if [[ ! -d "${django_dir}" ]]; then
         mkdir "${django_dir}"
         mkdir "${django_dir}/exports"
         mkdir "${django_dir}/imports"
         chown -R "${DEVKIT_USER}":"${DEVKIT_USER}" "${django_dir}"
         chmod -R 700 "${django_dir}"
    fi
}

function build_agent_plugins()
{
    # 需要安装到节点的插件目录
    local package_files=("diagnosis" "components" "doctor" "deploy/${IPERF_PACKAGE}" "deploy/${NET_FLOW_PACKAGE}" "deploy/agent_install_entry.sh" "deploy/agent_uninstall_entry.sh" "deploy/const.sh" "deploy/env_check.sh" "deploy/utils.sh" "deploy/doctor_cli.sh")
    # 需要从插件安装包中删除的文件
    # 这些日志配置文件中logPath写死，不能使用server上的相同配置
    local plugin_delete_files=("components/config/cfg.ini")
    local plugin_dir="${INSTALL_PATH}/devkitplugins/${PLUGIN}"
    local plugin_package_dir="${INSTALL_PATH}/tools/agent_install_packages/${PLUGIN}"
    if [[ -d "${plugin_package_dir}" ]]; then
        rm -rf "${plugin_package_dir}"
    fi
    mkdir "${plugin_package_dir}"
    chmod 700 "${plugin_package_dir}"
    if [[ ! -d "${plugin_dir}" ]]; then
        echo_and_log "Plugin ${PLUGIN} not exists" "${ERROR}" "${INSTALL_LOG_PATH}"
        exit "${ERROR}"
    else
        for package_file in "${package_files[@]}";
        do
            local src_file="${plugin_dir}/${package_file}"
            local dist_file="${plugin_package_dir}/${package_file}"
            local dist_dir
            dist_dir="$(dirname ${dist_file})"
            if [[ ! -d "${dist_dir}" ]]; then
                mkdir -p "${dist_dir}"
            fi
            if [[ -d "${src_file}" ]]; then
                if ! cp -rfp "${src_file}" "${dist_file}"; then
                    echo_and_log "Copy ${src_file} to ${dist_file} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
                    exit "${ERROR}"
                fi
            elif [[ -f "${src_file}" ]]; then
                if ! cp -fp "${src_file}" "${dist_file}"; then
                    echo_and_log "Copy ${src_file} to ${dist_file} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
                    exit "${ERROR}"
                fi
            fi
        done
    fi

    # 先删掉多余文件
    if [[ ! -d "${plugin_package_dir}" ]]; then
        echo_and_log "Pre-install Plugin ${PLUGIN} not exists" "${ERROR}" "${INSTALL_LOG_PATH}"
        exit "${ERROR}"
    else
        for delete_file in "${plugin_delete_files[@]}";
        do
            local delete_item="${plugin_package_dir}/${delete_file}"
            if [[ -d "${delete_item}" ]] && [[ ! -L "${delete_item}" ]]; then
                rm -rf "${delete_item}"
            elif [[ -f "${delete_item}" ]] && [[ ! -L "${delete_item}" ]]; then
                rm -f "${delete_item}"
            fi
            if [[ $? -ne 0 ]]; then
                echo_and_log "Delete ${delete_item} failed" "${ERROR}" "${INSTALL_LOG_PATH}"
                exit "${ERROR}"
            fi
        done
    fi
    local tar_dir
    tar_dir="$(dirname "${plugin_package_dir}")"
    local plugins_dir
    plugins_dir="$(basename "${plugin_package_dir}")"
    local plugins_tar_path="${tar_dir}/${PLUGIN}.tar.gz"
    if [[ -f "${plugins_tar_path}" ]]; then
        rm -f "${plugins_tar_path}"
    fi
    tar -czf "${plugins_tar_path}" -C "${tar_dir}" "${plugins_dir}"
    if [[ $? -eq 0 ]]; then
        cd "${tar_dir}" && rm -rf "./${plugins_dir}"
        chown root:devkit "${plugins_tar_path}"
        chmod 440 "${plugins_tar_path}"
        local sign_for_digest
        sign_for_digest=$(su - devkit -s /bin/bash -c "${PYTHON_PATH} -c \"import sys;sys.path.append('${INSTALL_PATH}/devkitplugins');from devkitcommon.common_utils.common_util import get_file_hash, sign_for_digest;print(sign_for_digest(get_file_hash('${plugins_tar_path}')))\"")

        if [[ -n "${sign_for_digest}" ]]; then
            local plugins_sign_path="${tar_dir}/${PLUGIN}.sign"
            echo "${sign_for_digest}" > "${plugins_sign_path}"
            chown root:devkit "${plugins_sign_path}"
            chmod 440 "${plugins_sign_path}"
        fi
        echo_and_log "Generate plugins agent package success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
    else
        echo_and_log "Generate plugins agent package failed" "${ERROR}" "${INSTALL_LOG_PATH}"
        exit "${ERROR}"
    fi
}

function main() {
    init_variable
    create_log "${INSTALL_LOG_PATH}"
    echo_and_log "Run sys_diagnosis install script" "${DOING}" "${INSTALL_LOG_PATH}"
    check_kunpeng "${INSTALL_LOG_PATH}"
    if [[ "${PLUGIN_FLAG}" == "web" ]]; then
        # 强制安装缺失依赖
        bash "${current_dir}/env_check.sh" "${INSTALL_PATH}" "${PLUGIN_FLAG}"
        local check_res="$?"
        if [[ -f "${current_dir}/make_env_check_result.py" ]]; then
            su - devkit -s /bin/bash -c "${PYTHON_PATH} ${current_dir}/make_env_check_result.py"
        else
            su - devkit -s /bin/bash -c "${PYTHON_PATH} ${current_dir}/make_env_check_result.pyc"
        fi
        local temp_env_json="${current_dir}/sys_diagnosis_env_res.json"
        local final_env_json_dir="${INSTALL_PATH}/devkitplugins/check_env_result"
        local final_env_json="${final_env_json_dir}/sys_diagnosis_env_res.json"
        if [[ -f "${temp_env_json}" ]] && [[ -d "${final_env_json_dir}" ]]; then
            mv "${temp_env_json}" "${final_env_json}"
            chmod 600 "${final_env_json}"
            chown "${DEVKIT_USER}":"${DEVKIT_USER}" "${final_env_json}"
        fi
        if [[ "${check_res}" -ne "${SUCCESS}" ]]; then
            echo_and_log "There are dependencies that do not meet the installation requirements" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        else
            echo_and_log "The environment check is passed" "${SUCCESS}" "${INSTALL_LOG_PATH}"
        fi
    else
        # 磁盘空间大小检测
        echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG_PATH}"
        disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
        if [[ "$?" -ne "${SUCCESS}" ]]; then
            echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG_PATH}"
            exit "${ERROR}"
        fi
        echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
    fi
    echo_and_log "Creating an Application Running User" "${DOING}" "${INSTALL_LOG_PATH}"
    create_launcher "${os_type}"
    echo_and_log "Creating an Application Running User" "${SUCCESS}" "${INSTALL_LOG_PATH}"
    copy_sudo_files "${INSTALL_PATH}" "${INSTALL_LOG_PATH}"
    add_sudoers "${INSTALL_PATH}" "${INSTALL_LOG_PATH}"
    install_third_app "${INSTALL_LOG_PATH}"
    cp "${IPERF_PACKAGE}" "${NET_FLOW_PACKAGE}" "${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy/"
    load_data
    create_export_import_dir
    copy_doctor_tool "${INSTALL_PATH}" "${INSTALL_LOG_PATH}"
    create_check_pd_result
    change_file_path_permission
    create_pid_path
    build_agent_plugins
    echo_and_log "Run sys_diagnosis install script success" "${SUCCESS}" "${INSTALL_LOG_PATH}"
}


main $@
